classdef Camera < handle
  % Camera
  %   The main interface to the depth cameras
  %
  % Methods:
  %   start - Start streaming of the camera
  %   stop - Stop streaming of the camera
  %   isStreaming - Check whether the camera is streaming
  %   isConnected - Check whether the camera is still connected
  %   getStreamList - Get the different output streams the camera can provide
  %   setStreamList - Set the streams that you want to get from the camera
  %   isStreamListSet - Check whether the stream list is set
  %   getSetStreamList - Get the stream list that is set on the camera
  %   getProcessConfig - Get the processing config of the camera
  %   setProcessConfig - Set the processing config of the camera
  %   hasFrames - Check if the camera has frames available
  %   getFrames - Get data frames from camera
  %   clearBuffer - Clear the circular buffer used to store frames
  %   getBufferSize - Get the circular buffer size
  %   setBufferSize - Set the circular buffer size
  %   getCameraConfig - Get the currently active camera config
  %   getSerial - Get the serial number of the current camera
  %   getCalibration - Get calibration from the camera
  %   setCalibration - Set calibration. If it is a csf camera, the calibration
  %                    will only be set in the pipeline. If it's a kea camera
  %                    the calibration will also be uploaded
  %   getUserPointerCapacity - Get the amount of user pointers that can be
  %                            stored
  %   setUserPointerCapacity - Set the amount of user pointers that can be
  %                            stored
  %   addUserPointer - Add a pointer that will be filled with the specified data
  %                    type. This method allows you to fill data straight into
  %                    your own data type without any additional copying.
  %   clearUserPointers - Clear all user pointers in memory
  %   configIndex - Get the currently set configuration index
  %   switchConfig - Switch to a different config in real time, depending on
  %                  configuration and camera, this can take 100-300ms
  %   possibleFrameTypes - Get the frame types that this camera can output
  %   outputFrameTypesSet - Whether output frame types have been set
  %   getOutputFrameTypes - Get the frame types the camera should output
  %   setOutputFrameTypes - Set the frame types the camera should output
  %   hasMultipleConfigurations - Check if multiple configurations are set.
  %                               Switch config can only be used when this is
  %                               the case.
  %   getMultipleCameraConfigurations - Get all the set camera configurations
  %   getMultipleProcessingConfigurations - Get all the set processing
  %                                         configurations
  %   getMultipleNames - Get the name for each configuration
  %   getMultipleDescriptions - Get the description for each configuration
  %

  properties(Access=protected)
    mPtr = uint64(0);
  end

  methods(Access=public)
    function obj = Camera(varargin)
      % Camera
      %   Construct new camera class
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        obj.mPtr = varargin{1};
      else
        error("Can't construct empty object");
      end
    end


    function delete(obj)
      if obj.mPtr ~= 0
        tof.mex('tof_camera_delete', obj.mPtr);
        obj.mPtr = 0;
      end
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function start(obj)
      % start
      %   Start streaming of the camera
      %

      tof.mex('tof_camera_start', obj.mPtr);
    end

    function stop(obj)
      % stop
      %   Stop streaming of the camera
      %

      tof.mex('tof_camera_stop', obj.mPtr);
    end

    function ret_val = isStreaming(obj)
      % isStreaming
      %   Check whether the camera is streaming
      %
      % Output:
      %   ret_val - logical - Is streaming
      %

      ret_val = tof.mex('tof_camera_is_streaming', obj.mPtr);
    end

    function ret_val = isConnected(obj)
      % isConnected
      %   Check whether the camera is still connected
      %
      % Output:
      %   ret_val - logical - Is connected
      %

      ret_val = tof.mex('tof_camera_is_connected', obj.mPtr);
    end

    function ret_val = getStreamList(obj)
      % getStreamList
      %   Get the different output streams the camera can provide
      %
      % Output:
      %   ret_val - vector[Stream] - The streams
      %

      ret_val = tof.mex('tof_camera_get_stream_list', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.Stream(vec_obj), ret_val);
    end

    function setStreamList(obj, streams)
      % setStreamList
      %   Set the streams that you want to get from the camera
      %
      % Inputs:
      %   streams - vector[Stream] - The wanted streams
      %

      tof.mex('tof_camera_set_stream_list', obj.mPtr, arrayfun(@(vec_obj)vec_obj.get_object_ptr(), streams));
    end

    function ret_val = isStreamListSet(obj)
      % isStreamListSet
      %   Check whether the stream list is set
      %
      % Output:
      %   ret_val - logical - Is stream list set
      %

      ret_val = tof.mex('tof_camera_is_stream_list_set', obj.mPtr);
    end

    function ret_val = getSetStreamList(obj)
      % getSetStreamList
      %   Get the stream list that is set on the camera
      %
      % Output:
      %   ret_val - vector[Stream] - The currently set streams
      %

      ret_val = tof.mex('tof_camera_get_set_stream_list', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.Stream(vec_obj), ret_val);
    end

    function ret_val = getProcessConfig(obj)
      % getProcessConfig
      %   Get the processing config of the camera
      %
      % Output:
      %   ret_val - ProcessingConfig - The processing config
      %

      ret_val = tof.mex('tof_camera_get_process_config', obj.mPtr);
      ret_val = tof.ProcessingConfig(ret_val);
    end

    function setProcessConfig(obj, config)
      % setProcessConfig
      %   Set the processing config of the camera
      %
      % Inputs:
      %   config - ProcessingConfig - The processing config
      %

      tof.mex('tof_camera_set_process_config', obj.mPtr, config.get_object_ptr());
    end

    function ret_val = hasFrames(obj)
      % hasFrames
      %   Check if the camera has frames available
      %
      % Output:
      %   ret_val - logical - Has frames available
      %

      ret_val = tof.mex('tof_camera_has_frames', obj.mPtr);
    end

    function ret_val = getFrames(obj)
      % getFrames
      %   Get data frames from camera
      %
      % Output:
      %   ret_val - vector[Data] - Data frames
      %

      ret_val = tof.mex('tof_camera_get_frames', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.Data(vec_obj), ret_val);
    end

    function clearBuffer(obj)
      % clearBuffer
      %   Clear the circular buffer used to store frames
      %

      tof.mex('tof_camera_clear_buffer', obj.mPtr);
    end

    function ret_val = getBufferSize(obj)
      % getBufferSize
      %   Get the circular buffer size
      %
      % Output:
      %   ret_val - numeric - The maximum number of sets of frames that is
      %                       stored before the oldest is overwritten
      %

      ret_val = tof.mex('tof_camera_get_buffer_size', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setBufferSize(obj, size)
      % setBufferSize
      %   Set the circular buffer size
      %
      % Inputs:
      %   size - numeric - The maximum number of sets of frames that is stored
      %                    before the oldest is overwritten
      %

      tof.mex('tof_camera_set_buffer_size', obj.mPtr, size);
    end

    function ret_val = getCameraConfig(obj)
      % getCameraConfig
      %   Get the currently active camera config
      %
      % Output:
      %   ret_val - CameraConfig - The camera config
      %

      ret_val = tof.mex('tof_camera_get_camera_config', obj.mPtr);
      ret_val = tof.CameraConfig(ret_val);
    end

    function ret_val = getSerial(obj)
      % getSerial
      %   Get the serial number of the current camera
      %
      % Output:
      %   ret_val - string - The serial
      %

      ret_val = tof.mex('tof_camera_get_serial', obj.mPtr);
    end

    function ret_val = getCalibration(obj)
      % getCalibration
      %   Get calibration from the camera
      %
      % Output:
      %   ret_val - Calibration - The calibration
      %

      ret_val = tof.mex('tof_camera_get_calibration', obj.mPtr);
      ret_val = tof.Calibration(ret_val);
    end

    function ret_val = getUserPointerCapacity(obj)
      % getUserPointerCapacity
      %   Get the amount of user pointers that can be stored
      %
      % Output:
      %   ret_val - numeric - The capacity
      %

      ret_val = tof.mex('tof_camera_get_user_pointer_capacity', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setUserPointerCapacity(obj, capacity)
      % setUserPointerCapacity
      %   Set the amount of user pointers that can be stored
      %
      % Inputs:
      %   capacity - numeric - The capacity
      %

      tof.mex('tof_camera_set_user_pointer_capacity', obj.mPtr, capacity);
    end

    function clearUserPointers(obj)
      % clearUserPointers
      %   Clear all user pointers in memory
      %

      tof.mex('tof_camera_clear_user_pointers', obj.mPtr);
    end

    function ret_val = configIndex(obj)
      % configIndex
      %   Get the currently set configuration index
      %
      % Output:
      %   ret_val - numeric - config index
      %

      ret_val = tof.mex('tof_camera_config_index', obj.mPtr);
      ret_val = double(ret_val);
    end

    function switchConfig(obj, config_index)
      % switchConfig
      %   Switch to a different config in real time, depending on configuration
      %   and camera, this can take 100-300ms
      %
      % Inputs:
      %   config_index - numeric - Config index to switch to
      %

      tof.mex('tof_camera_switch_config', obj.mPtr, config_index);
    end

    function ret_val = possibleFrameTypes(obj)
      % possibleFrameTypes
      %   Get the frame types that this camera can output
      %
      % Output:
      %   ret_val - vector[FrameType] - Possible frame types
      %

      ret_val = tof.mex('tof_camera_possible_frame_types', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = outputFrameTypesSet(obj)
      % outputFrameTypesSet
      %   Whether output frame types have been set
      %
      % Output:
      %   ret_val - logical - Output frame types set
      %

      ret_val = tof.mex('tof_camera_output_frame_types_set', obj.mPtr);
    end

    function ret_val = getOutputFrameTypes(obj)
      % getOutputFrameTypes
      %   Get the frame types the camera should output
      %
      % Output:
      %   ret_val - vector[FrameType] - Output frame types
      %

      ret_val = tof.mex('tof_camera_get_output_frame_types', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setOutputFrameTypes(obj, output_frame_types)
      % setOutputFrameTypes
      %   Set the frame types the camera should output
      %
      % Inputs:
      %   output_frame_types - vector[FrameType] - Output frame types
      %

      tof.mex('tof_camera_set_output_frame_types', obj.mPtr, uint32(output_frame_types));
    end

    function ret_val = hasMultipleConfigurations(obj)
      % hasMultipleConfigurations
      %   Check if multiple configurations are set. Switch config can only be
      %   used when this is the case.
      %
      % Output:
      %   ret_val - logical - Is set
      %

      ret_val = tof.mex('tof_camera_has_multiple_configurations', obj.mPtr);
    end

    function ret_val = getMultipleCameraConfigurations(obj)
      % getMultipleCameraConfigurations
      %   Get all the set camera configurations
      %
      % Output:
      %   ret_val - vector[CameraConfig] - Multiple camera configurations
      %

      ret_val = tof.mex('tof_camera_get_multiple_camera_configurations', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.CameraConfig(vec_obj), ret_val);
    end

    function ret_val = getMultipleProcessingConfigurations(obj)
      % getMultipleProcessingConfigurations
      %   Get all the set processing configurations
      %
      % Output:
      %   ret_val - vector[ProcessingConfig] - Multiple processing
      %                                        configurations
      %

      ret_val = tof.mex('tof_camera_get_multiple_processing_configurations', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.ProcessingConfig(vec_obj), ret_val);
    end

    function ret_val = getMultipleNames(obj)
      % getMultipleNames
      %   Get the name for each configuration
      %
      % Output:
      %   ret_val - vector[string] - Names
      %

      ret_val = tof.mex('tof_camera_get_multiple_names', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getMultipleDescriptions(obj)
      % getMultipleDescriptions
      %   Get the description for each configuration
      %
      % Output:
      %   ret_val - vector[string] - Descriptions
      %

      ret_val = tof.mex('tof_camera_get_multiple_descriptions', obj.mPtr);
      ret_val = double(ret_val);
    end

  end
end
